// +build win

package mxnet2api

//********************
// Windows用ファイル
//********************
// 64ビット Windows で利用してください。
// 64ビット版 MingWをインストールする必要があります
//
// 64ビット MinGWインストール方法
// https://qiita.com/Marny/items/133e7032c3d20d2e77b1
//

/*
#cgo LDFLAGS:  -L . -l mxnet2api
#include <stdlib.h>
#include "mxnet2api.h"
*/
import (
	"C"
)

import "unsafe"

func Init_MatrixAPI() (ret int16) {

	ret = int16(C.rInit_MatrixAPI())
	return
}

func Release_MatrixAPI() (ret int16) {

	ret = int16(C.rRelease_MatrixAPI())
	return
}

func GetVersionAPI() (ret int32) {
	ret = int32(C.rGetVersionAPI())
	return
}

func GetVersionDRV() (ret int32) {
	ret = int32(C.rGetVersionDRV())
	return
}

func GetVersionDRV_USB() (ret int32) {
	ret = int32(C.rGetVersionDRV_USB())
	return
}

func Dongle_Version(DngNr int16) (ret int32) {
	ret = int32(C.rDongle_Version(C.short(DngNr), 85))
	return
}

func Dongle_Model(DngNr int16) (ret int32) {
	ret = int32(C.rDongle_Model(C.short(DngNr), 85))
	return
}

func Dongle_MemSize(DngNr int16) (ret int16) {
	ret = int16(C.rDongle_MemSize(C.short(DngNr), 85))
	return
}

func Dongle_Count() (ret int16) {

	ret = int16(C.rDongle_Count(85))
	return
}

func Dongle_ReadSerNr(UserCode int32, DngNr int16) (ret int32) {

	ret = int32(C.rDongle_ReadSerNr(C.int(UserCode), C.short(DngNr), 85))
	return
}

func Dongle_ReadData(UserCode int32, Data []int32, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.rDongle_ReadData(C.int(UserCode), (*C.int)(&Data[0]), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_ReadDataEx(UserCode int32, Data []int32, Fpos int16, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.rDongle_ReadDataEx(C.int(UserCode), (*C.int)(&Data[0]), C.short(Fpos), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_GetKeyFlag(UserCode int32, DngNr int16) (ret int16) {
	ret = int16(C.rDongle_GetKeyFlag(C.int(UserCode), C.short(DngNr), 85))
	return
}

func Dongle_EncryptData(UserCode int32, DataBlock []int32, DngNr int16) (ret int16) {
	ret = int16(C.rDongle_EncryptData(C.int(UserCode), (*C.int)(&DataBlock[0]), C.short(DngNr), 85))
	return
}

func Dongle_DecryptData(UserCode int32, DataBlock []int32, DngNr int16) (ret int16) {
	ret = int16(C.rDongle_DecryptData(C.int(UserCode), (*C.int)(&DataBlock[0]), C.short(DngNr), 85))
	return
}


func Dongle_GetDriverFlag(UserCode int32, DngNr int16) (ret int16) {
	ret = int16(C.rDongle_GetDriverFlag(C.int(UserCode), C.short(DngNr), 85))
	return
}

func SetConfig_MatrixNet(nAccess int16, nFile string) (ret int16) {
	serverFile := C.CString(nFile)
	ret = int16(C.rSetConfig_MatrixNet(C.short(nAccess), serverFile))
	C.free(unsafe.Pointer(serverFile))
	return
}

func GetConfig_MatrixNet(Category int16) (ret int32) {
	ret = int32(C.rGetConfig_MatrixNet(C.short(Category)))
	return
}

func LogIn_MatrixNet(UserCode int32, AppSlot int16, DngNr int16) (ret int16) {
	ret = int16(C.rLogIn_MatrixNet(C.int(UserCode), C.short(AppSlot), C.short(DngNr)))
	return
}

func LogOut_MatrixNet(UserCode int32, AppSlot int16, DngNr int16) (ret int16) {
	ret = int16(C.rLogOut_MatrixNet(C.int(UserCode), C.short(AppSlot), C.short(DngNr)))
	return
}

func MxApp_Encrypt(Data *[2]uint32, Key [4]uint32) int16 {
	var delta, sum uint32
	var cnt int16

	sum = 0
	delta = 0x9E3779B9
	cnt = 32
	for cnt = 32; cnt > 0; cnt-- {
		Data[0] += ((Data[1]<<4 ^ Data[1]>>5) + Data[1]) ^ (sum + Key[sum&3])
		sum += delta
		Data[1] += ((Data[0]<<4 ^ Data[0]>>5) + Data[0]) ^ (sum + Key[sum>>11&3])
	}
	return 0
}

func MxApp_Decrypt(Data *[2]uint32, Key [4]uint32) int16 {
	var delta, sum uint32
	var cnt int16

	sum = 0xC6EF3720
	delta = 0x9E3779B9
	cnt = 32
	for cnt = 32; cnt > 0; cnt-- {
		Data[1] -= ((Data[0]<<4 ^ Data[0]>>5) + Data[0]) ^ (sum + Key[sum>>11&3])
		sum -= delta
		Data[0] -= ((Data[1]<<4 ^ Data[1]>>5) + Data[1]) ^ (sum + Key[sum&3])
	}
	return 0
}

func MxNet_SetIP(ip string) (setip string){
	setip = C.GoString(C.MxNet_SetIPA( C.CString(ip)))
	return
}

func MxNet_SetPort(port int32){
	_ = C.MxNet_SetPort( C.int(port))
	return
}