package mxnet2license

import (
	"fmt"
	"time"
	"os"
	"os/signal"
	"syscall"
	"bufio"
	"example.com/mxnet2api"
)

var channel = make(chan int)
var	sigs = make(chan os.Signal, 1)
var	done = make(chan bool, 1)

var ip string="mxnet2b.local"
var port int32 = 12300

var UserCode int32 = 1234
var appSlot int16 = 7
var interval int16 = 5

var mxnet2Connected bool = false

func getLicense(c chan int){

//
// mxnetapi.ini設定の上書
//
//	var setip=mxnet2api.MxNet_SetIP(ip)
//	fmt.Println("setip=", setip)
//	if setip == "" {
//		fmt.Println("*** SetIP Error. IP will be read from mxnetapi.ini ***")
//	}
//	mxnet2api.MxNet_SetPort(port)

	var ret int16 = mxnet2api.Init_MatrixAPI()
	if ret < 0	{
		fmt.Println("          Init Error;", ret)
		c <- -1
		return
	}

	mxnet2Connected=true

	ret = mxnet2api.Dongle_Count()
	fmt.Println("          count=", ret)

	if ret < 1 {
		fmt.Println("          Connect a dongle")
		c <- -2
		return
	}

	var serNr int32 = mxnet2api.Dongle_ReadSerNr(UserCode, 1)
	fmt.Println("          SerNr=", serNr)
	if serNr <= 0 {
		c <- -3
		return
	}

	// ***** read memory **********
	var FieldData = make([]int32, 10)
	ret = mxnet2api.Dongle_ReadDataEx(UserCode, FieldData, 5, 10, 1)
	fmt.Println("          ReadDataEx=", ret)
	if ret < 1 {
		c <- -4
		return
	}

	for idx, data := range FieldData {
		fmt.Println("          idx=", idx, "Data=", uint32(data))
	}

	ret = mxnet2api.LogIn_MatrixNet(UserCode, appSlot, 1)
	fmt.Println("          Login=", ret)
	if ret < 0 {
		c <- -5
		return
	}

	c <- int(ret)
}

func AppExit(){

	if mxnet2Connected ==true {
		fmt.Println("released")
		mxnet2api.Release_MatrixAPI()
	}
	fmt.Println("appExit:program exiting...\n")
	os.Exit(0)
}

func loginUpdate( done chan bool ) {

	for{
		time.Sleep(time.Duration(interval) * time.Second)

		var ret int16 = mxnet2api.LogIn_MatrixNet(UserCode, appSlot, 1)
		fmt.Println("          Login=", ret)
		if ret < 0 {
			fmt.Println("          ***Login Error***")
			done <- true
			return
		}

	}

}

func waitForKey( done chan bool ){

	consoleReader := bufio.NewReaderSize(os.Stdin, 1)

	for{
		input, _ := consoleReader.ReadByte()
		ascii := input

		if ascii == 13 {
			fmt.Println("          Exiting...")
			done <- true
			break
		}
	}

}

func waitForSignal(sigs chan os.Signal, done chan bool){
	sig := <-sigs
	fmt.Println( "******", sig, "******" )
	done <- true
}

func waitForTermination(done chan bool){
	<- done
	AppExit()
}


func SetupLicense(){

	go getLicense(channel)

	//wait to acquire license
	var result int = <-channel

	if result < 0{
		fmt.Println("          failed to acquire license")
		AppExit()
		return;
	}

	signal.Notify(sigs, syscall.SIGINT, syscall.SIGTERM)

	go waitForSignal(sigs, done)
//	go waitForKey(done)
	go loginUpdate(done)
	go waitForTermination(done)

}

